//
//  MessagesViewController.swift
//  MessagesExtension
//
//  Created by Mohammad Azam on 10/2/16.
//  Copyright © 2016 Mohammad Azam. All rights reserved.
//

import UIKit
import Messages

class MessagesViewController: MSMessagesAppViewController,RSVPCompactViewControllerDelegate, CreateRSVPViewControllerDelegate, RSVPSelectionViewControllerDelegate  {
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
    }
    
    private func presentViewController(with conversation:MSConversation, for presentationStyle :MSMessagesAppPresentationStyle) {
        
        var controller :UIViewController!
        
        if presentationStyle == .compact {
            
            controller = instantiateRSVPCompactViewController()
            
        } else {
            
            if conversation.selectedMessage != nil {
                
                let rsvp = RSVP(message: conversation.selectedMessage!)
                controller = instantiateRSVPSelectionViewController(rsvp: rsvp!)
            } else {
                 controller = instantiateCreateRSVPViewController()
            }
            
        }
        
        // Remove any existing child controllers.
        for child in childViewControllers {
            child.willMove(toParentViewController: nil)
            child.view.removeFromSuperview()
            child.removeFromParentViewController()
        }
        
        addChildViewController(controller)
        
        controller.view.frame = self.view.bounds
        controller.view.translatesAutoresizingMaskIntoConstraints = false
        self.view.addSubview(controller.view)
        
        controller.view.leftAnchor.constraint(equalTo: view.leftAnchor).isActive = true
        controller.view.rightAnchor.constraint(equalTo: view.rightAnchor).isActive = true
        controller.view.topAnchor.constraint(equalTo: view.topAnchor).isActive = true
        controller.view.bottomAnchor.constraint(equalTo: view.bottomAnchor).isActive = true
        
        controller.didMove(toParentViewController: self)
    }
    
    func rsvpSelectionViewControllerDidSelectYes(rsvp: RSVP) {
       
        requestPresentationStyle(.compact)
        let message = composeMessage(rsvp :rsvp)
        self.activeConversation?.insert(message, completionHandler: nil)
    }
    
    func rsvpSelectionViewControllerDidSelectNo(rsvp: RSVP) {
        
        requestPresentationStyle(.compact)
        let message = composeMessage(rsvp :rsvp)
        self.activeConversation?.insert(message, completionHandler: nil)
    }
    
    func rsvpCompactViewControllercreateRSVP() {
        
        requestPresentationStyle(.expanded)
    }
    
    func createRSVPViewControllerDidCreatedRSVP(rsvp: RSVP) {
        
        requestPresentationStyle(.compact)
        
        let message = composeMessage(rsvp :rsvp)
        self.activeConversation?.insert(message, completionHandler: nil)
    }
    
    override func willSelect(_ message: MSMessage, conversation: MSConversation) {
        
        
    }
    
    override func didSelect(_ message: MSMessage, conversation: MSConversation) {
        
    }
    
    private func composeMessage(rsvp :RSVP) -> MSMessage {
        
        let session = self.activeConversation?.selectedMessage?.session
        
        let message = MSMessage(session: session ?? MSSession())
        
        let components = NSURLComponents()
        
        components.queryItems = [URLQueryItem(name: "title", value:
            rsvp.title),URLQueryItem(name:"yesCount",value:"\(rsvp.yes)"),
                        URLQueryItem(name:"noCount",value:"\(rsvp.no)")]
        
      //  "title=ARE YOU COMING TO MY PARTY&yesCount=0&noCount=0"
        
        message.url = components.url!
        
        let layout = MSMessageTemplateLayout()
        layout.image = UIImage(named: "partyballoons.jpg")
        layout.caption = rsvp.title
        layout.subcaption = "Yes Count: \(rsvp.yes)"
        layout.trailingSubcaption = "No Count:\(rsvp.no)"
        
        message.layout = layout
        
        return message
        
    }
    
    private func instantiateRSVPSelectionViewController(rsvp :RSVP) -> UIViewController {
        
        guard let controller = self.storyboard?.instantiateViewController(withIdentifier: "RSVPSelectionViewController") as? RSVPSelectionViewController else {
            fatalError("RSVPSelectionViewController not found")
        }
        
        controller.rsvp = rsvp
        controller.delegate = self
        
        return controller
    }
    
    private func instantiateCreateRSVPViewController() -> UIViewController {
        
        guard let controller = self.storyboard?.instantiateViewController(withIdentifier: "CreateRSVPViewController") as? CreateRSVPViewController else {
            fatalError("CreateRSVPViewController not found")
        }
        
        controller.delegate = self

        return controller
    }
    
    private func instantiateRSVPCompactViewController() -> UIViewController {
        
        guard let controller = self.storyboard?.instantiateViewController(withIdentifier: "RSVPCompactViewController") as? RSVPCompactViewController else {
            fatalError("RSVPCompactViewController not found")
        }
        
        controller.delegate = self
        return controller
    }
    
    override func willBecomeActive(with conversation: MSConversation) {
        presentViewController(with: conversation, for: presentationStyle)
    }
    
    override func didTransition(to presentationStyle: MSMessagesAppPresentationStyle) {
        
        guard let conversation = self.activeConversation else {
            fatalError("No active conversation")
        }
        
        presentViewController(with: conversation, for: presentationStyle)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    
    
}
